@@ -16,6 +16,7 @@
 
 from __future__ import print_function
 
+import logging
 import optparse
 import os
 import StringIO
@@ -31,6 +32,8 @@ import meld.ui.util
 
 from meld.conf import _
 
+log = logging.getLogger(__name__)
+
 
 class MeldApp(Gtk.Application):
 
@@ -84,7 +87,7 @@ class MeldApp(Gtk.Application):
         window = self.get_active_window().meldwindow
         if not window.has_pages():
             window.append_new_comparison()
-        window.widget.show()
+        window.widget.present()
         return 0
 
     def do_window_removed(self, widget):
@@ -133,13 +136,20 @@ class MeldApp(Gtk.Application):
     def get_meld_window(self):
         return self.get_active_window().meldwindow
 
-    def open_paths(self, paths, **kwargs):
+    def open_files(self, files, **kwargs):
         new_tab = kwargs.pop('new_tab')
         if new_tab:
             window = self.get_meld_window()
         else:
             window = self.new_window()
-        return window.open_paths(paths, **kwargs)
+
+        paths = [f.get_path() for f in files]
+        try:
+            return window.open_paths(paths, **kwargs)
+        except ValueError:
+            if not new_tab:
+                self.remove_window(window.widget)
+            raise
 
     def diff_files_callback(self, option, opt_str, value, parser):
         """Gather --diff arguments and append to a list"""
@@ -180,9 +190,10 @@ class MeldApp(Gtk.Application):
                 self.command_line = command_line
                 self.should_exit = False
                 self.output = StringIO.StringIO()
+                self.exit_status = 0
                 optparse.OptionParser.__init__(self, *args, **kwargs)
 
-            def exit(self, *args):
+            def exit(self, status=0, msg=None):
                 self.should_exit = True
                 # FIXME: This is... let's say... an unsupported method. Let's
                 # be circumspect about the likelihood of this working.
@@ -191,6 +202,7 @@ class MeldApp(Gtk.Application):
                         self.command_line, self.output.getvalue())
                 except:
                     print(self.output.getvalue())
+                self.exit_status = status
 
             def print_usage(self, file=None):
                 if self.usage:
@@ -203,6 +215,16 @@ class MeldApp(Gtk.Application):
             def print_help(self, file=None):
                 print(self.format_help(), file=self.output)
 
+            def error(self, msg):
+                self.local_error(msg)
+                raise ValueError()
+
+            def local_error(self, msg):
+                self.print_usage()
+                error_string = _("Error: %s\n") % msg
+                print(error_string, file=self.output)
+                self.exit(2)
+
         parser = GLibFriendlyOptionParser(
             command_line=command_line,
             usage=usage,
@@ -236,22 +258,34 @@ class MeldApp(Gtk.Application):
             dest="diff", default=[],
             help=_("Create a diff tab for the supplied files or folders"))
 
-        rawargs = command_line.get_arguments()[1:]
-        options, args = parser.parse_args(rawargs)
-        parser.command_line = None
-        if parser.should_exit:
-            if command_line.get_is_remote():
-                return 0
-            else:
+        def cleanup():
+            if not command_line.get_is_remote():
                 self.quit()
+            parser.command_line = None
+
+        rawargs = command_line.get_arguments()[1:]
+        try:
+            options, args = parser.parse_args(rawargs)
+        except ValueError:
+            # Thrown to avert further parsing when we've hit an error, because
+            # of our weird when-to-exit issues.
+            pass
+
+        if parser.should_exit:
+            cleanup()
+            return parser.exit_status
 
         if len(args) > 3:
-            parser.error(_("too many arguments (wanted 0-3, got %d)") %
-                         len(args))
+            parser.local_error(_("too many arguments (wanted 0-3, got %d)") %
+                               len(args))
         elif options.auto_merge and len(args) < 3:
-            parser.error(_("can't auto-merge less than 3 files"))
+            parser.local_error(_("can't auto-merge less than 3 files"))
         elif options.auto_merge and any([os.path.isdir(f) for f in args]):
-            parser.error(_("can't auto-merge directories"))
+            parser.local_error(_("can't auto-merge directories"))
+
+        if parser.should_exit:
+            cleanup()
+            return parser.exit_status
 
         if options.comparison_file or (len(args) == 1 and
                                        args[0].endswith(".meldcmp")):
@@ -261,33 +295,44 @@ class MeldApp(Gtk.Application):
             try:
                 tab = self.get_meld_window().append_recent(gio_file.get_uri())
             except (IOError, ValueError):
-                parser.error(_("Error reading saved comparison file"))
+                parser.local_error(_("Error reading saved comparison file"))
+            if parser.should_exit:
+                cleanup()
+                return parser.exit_status
             return tab
 
+        tab = None
         error = None
         comparisons = options.diff + [args]
         options.newtab = options.newtab or not command_line.get_is_remote()
         for i, paths in enumerate(comparisons):
+            files = [command_line.create_file_for_arg(p) for p in paths]
             try:
-                tab = self.open_paths(
-                    paths, auto_compare=options.auto_compare,
+                tab = self.open_files(
+                    files, auto_compare=options.auto_compare,
                     auto_merge=options.auto_merge, new_tab=options.newtab,
                     focus=i == 0)
             except ValueError as err:
                 error = err
+            else:
+                if options.label:
+                    tab.set_labels(options.label)
 
-            if options.label:
-                tab.set_labels(options.label)
-
-            if options.outfile and isinstance(tab, filediff.FileDiff):
-                tab.set_merge_output_file(options.outfile)
+                if options.outfile and isinstance(tab, filediff.FileDiff):
+                    tab.set_merge_output_file(options.outfile)
 
         if error:
-            if not self.get_meld_window().has_pages():
-                parser.error(error)
+            log.debug("Couldn't open comparison: %s", error)
+            if not tab:
+                parser.local_error(error)
             else:
                 print(error)
 
+        if parser.should_exit:
+            cleanup()
+            return parser.exit_status
+
+        parser.command_line = None
         return tab if len(comparisons) == 1 else None
 
 
