@@ -149,6 +149,39 @@ class MeldWindow(gnomeglade.Component):
         self.ui = Gtk.UIManager()
         self.ui.insert_action_group(self.actiongroup, 0)
         self.ui.add_ui_from_file(ui_file)
+
+        # Manually handle shells that don't show an application menu
+        gtk_settings = Gtk.Settings.get_default()
+        if not gtk_settings.props.gtk_shell_shows_app_menu:
+            from meldapp import app
+
+            def make_app_action(name):
+                def app_action(*args):
+                    app.lookup_action(name).activate(None)
+                return app_action
+
+            app_actions = (
+                ("AppMenu", None, _("_Meld")),
+                ("Quit", Gtk.STOCK_QUIT, None, None, _("Quit the program"),
+                 make_app_action('quit')),
+                ("Preferences", Gtk.STOCK_PREFERENCES, _("Prefere_nces"), None,
+                 _("Configure the application"),
+                 make_app_action('preferences')),
+                ("Help", Gtk.STOCK_HELP, _("_Contents"), "F1",
+                 _("Open the Meld manual"), make_app_action('help')),
+                ("About", Gtk.STOCK_ABOUT, None, None,
+                 _("About this application"), make_app_action('about')),
+            )
+
+            app_actiongroup = Gtk.ActionGroup(name="AppActions")
+            app_actiongroup.set_translation_domain("meld")
+            app_actiongroup.add_actions(app_actions)
+            self.ui.insert_action_group(app_actiongroup, 0)
+
+            ui_file = gnomeglade.ui_file("appmenu-fallback.xml")
+            self.ui.add_ui_from_file(ui_file)
+            self.widget.set_show_menubar(False)
+
         self.tab_switch_actiongroup = None
         self.tab_switch_merge_id = None
 
@@ -210,6 +243,9 @@ class MeldWindow(gnomeglade.Component):
         self.scheduler.connect("runnable", self.on_scheduler_runnable)
         window_size = settings.get_value('window-size')
         self.widget.set_default_size(window_size[0], window_size[1])
+        window_state = settings.get_string('window-state')
+        if window_state == 'maximized':
+            self.widget.maximize()
         self.ui.ensure_update()
         self.diff_handler = None
         self.undo_handlers = tuple()
@@ -385,6 +421,10 @@ class MeldWindow(gnomeglade.Component):
             variant = GLib.Variant('(ii)', (event.width, event.height))
             settings.set_value('window-size', variant)
 
+        maximised = state & Gdk.WindowState.MAXIMIZED
+        window_state = 'maximized' if maximised else 'normal'
+        settings.set_string('window-state', window_state)
+
     def on_menu_file_new_activate(self, menuitem):
         self.append_new_comparison()
 
